import sys
import time
import random
import itertools
import traceback
import shutil
from os.path import dirname, join, isfile, isdir, realpath
from os import getcwd, chdir
from ConfigParser import ConfigParser

from handcrank import version
from handcrank import generator
from handcrank.dirchange import Monitor

ascii_header = r"""
__               .--------.
   `.      ______|        |\
    |     / ```` |        | |
    |     |  ,---|        |/      
    |_____|  |   `--------'         
    | ````   |                    
    |~~~~~~~/ 
___.'"""


class MissingConfigFile(Exception):
    """
    Raised when the configuration file for a site cannot be found or could not
    be opened
    """
    pass


class Runner(object):
    """
    Responsible for doing a majority of the work in the system.  It's the
    clearing house of activity, passing control to a generator or building a
    new site from a template.
    """
    # Little messages that occur whenever the handle is cranked
    crank_list = ('click click click', 'squeak spueak spueak',
        'crank crank crank', 'erkkk erkkk', 'round it goes',
        'cranking the handle', 'is that smoke?', 'chugga chugga',
        'zzzzzzzzrrrrrazzz', 'scratchy clicky erkkkk',)
    rotated_cranks = random.sample(crank_list, len(crank_list))

    def __init__(self, options):
        self.options = options

        print ascii_header
        print "\n\nHandcrank version %s" % version.number

    def begin(self):
        """
        Performs the action specified and begins the actual work

        This can be any of the `actions` parser arguments.  See
        handcrank/__init__.py for a list of what those can be.
        """
        getattr(self, self.options.actions)()

    def get_delegate_class(self, delegate_cls, from_dir):
        """
        A delegate is able to alter the behavior of the generator.

        This method tries to import delegate_cls from the `delegate` module
        within from_dir (meaning there needs to be a delegate.py in there), if
        it can't succeed it returns a class that essentially does nothing but
        provides all the necessary methods.

        Some of the things it can do:
            * Change the order of the source files so they appear in the
              generated output in a different order
            * Add extra context to the Template when it renders (for example,
              adding a variable called 'last10tweets' that are...well I'm sure
              you can figure out what that is.
        """
        try:
            if from_dir not in sys.path:
                sys.path.append(from_dir)
            dlg = __import__('delegate', globals(), locals(), [delegate_cls])
            return getattr(dlg, delegate_cls)
        except ImportError as ie:
            return generator.DelegateBase

    def build_generator_options(self, config_file):
        """
        When the generate command is called, this takes a ConfigParser object
        and generates the options for our Generator.

        The reason that this is necessary is that our config file has relative
        paths inside of it.  So an important step is to expand those relative
        paths into real paths so the generator doesn't have to it.

        This returns a dictionary that is suitable for creating a Generator
        object using the **gen_options syntax.
        """
        config = ConfigParser()
        try:
            config.readfp(open(config_file))
        except IOError:
            raise MissingConfigFile('Could not find %s' % config_file)

        options = {
            'from_dir': config.get('source', 'directory'),
            'tmp_dir': config.get('template', 'directory'),
            'out_dir': config.get('output', 'directory')}

        for d in options:
            dir = options[d]
            if not dir.startswith('/'):
                options[d] = realpath(join(dirname(config_file), dir))

        options.update({
            'template_file': config.get('template', 'name'),
            'output_file': config.get('output', 'name')})

        return options

    def generate(self):
        """
        This command is used when the user has a site directory with
        source files and a template that they wish to create a generated
        output.

        The basic idea is that a set of reStructuredText files become HTML
        files with possibly Javascript and CSS to go along with it.

        The generated output can then be opened in a browser locally and
        reviewed.
        """
        if self.options.site_dir:
            try:
                chdir(self.options.site_dir)
            except OSError:
                print("Cannot access %s" % self.options.site_dir)
                print("Do you mean to run `handcrank --sitedir %s "
                      "startsite`?" % self.options.site_dir)
                sys.exit(1)

        self.config_file = join(getcwd(), 'config.cfg')

        gen_options = self.build_generator_options(self.config_file)

        if 'delegate' in gen_options:
            cls_str = gen_options['delegate']
        else:
            cls_str = 'Delegate'

        delegate_cls = self.get_delegate_class(cls_str,
            dirname(self.config_file))

        def inner_run(gen_options):
            """
            Used by both the single run generate and the daemon mode to generate
            the site, it uses the generator class and the options that were
            assembled from the config file to create a output directory that
            represents the generated site.

            After it successfully runs, it prints a report letting the user
            know some information about what was generated and where it ended
            up.
            """
            gen = generator.OnePageGenerator(**gen_options)
            gen.delegate = delegate_cls()

            report = gen.turn_handle()
            print ' %s' % messages.next()
            report.dump()

        def inner_loop(gen_options):
            """
            This loop continually runs until the user breaks it with a CTRL-C
            or other KeyboardInterrupt raising event.

            This acts similar to a daemon.  It will continually monitor the
            source and template directories and a change in those files will
            result in the inner_run being called.
            """
            while True:
                try:
                    if has_changed():
                        inner_run(gen_options)
                    time.sleep(0.5)
                except KeyboardInterrupt:
                    sys.exit(0)
                except Exception as e:
                    print traceback.format_exc()

        def has_changed():
            """
            We need to monitor the template directory and the source
            directories for change.  If either one of these do, we return True
            and that can let the generator then re-generate the site content to
            the output directory.
            """
            return from_monitor.has_changed() or tmp_monitor.has_changed()

        messages = itertools.cycle(self.rotated_cranks)

        inner_run(gen_options)

        if self.options.daemon:
            print "Running in daemon mode, hit <CTRL-C> to exit"
            from_monitor = Monitor(gen_options['from_dir'],
                ignore_directory=[gen_options['out_dir']])
            tmp_monitor = Monitor(gen_options['tmp_dir'],
                ignore_directory=[gen_options['out_dir']])
            inner_loop(gen_options)

    def startsite(self):
        """
        Creates a site directory out of an internal template that handcrank
        has.  This saves the user some time as they do not have to create the
        files manully.  It's only argument, which is required, is the site_dir
        in which it will create the new directory.

        If the directory exists, this command will fail as it would be very
        uncommon to want to overwrite an existing directory.

        This command is handy for beginning a new project, but not much use
        after that.
        """
        site_dir = self.options.site_dir

        if not site_dir:
            print('Whoops, specify --sitedir to tell us where '
                'to create your new site')
            sys.exit(1)

        if isdir(site_dir):
            print('Whoa there, this directory already exists; refusing to '
                'clobber it because it\'s most likely not what you want. '
                'You delete it if that\'s what you really need.')
            sys.exit(1)

        skeleton = join(dirname(__file__), 'skeleton')
        shutil.copytree(skeleton, site_dir)

        print 'Created site here: %s' % site_dir
        print 'Run `handbrack --sitedir %s generate` to get cranking' % site_dir

        sys.exit(0)



import sys

try:
    import argparse

    parser = argparse.ArgumentParser(prog='handcrank',
        description='Hand crank static site generator using reStructuredText')
    parser.add_argument('actions', choices=('generate', 'startsite',),
        help='`generate` will create html from the rST files, `startsite` will '
        'build a new site directory you can start with')
    parser.add_argument('--keepgoing', dest='daemon', action='store_true',
        help='Runs a daemon, re-generating when files change',
        default=False)
    parser.add_argument('--sitedir', dest='site_dir', metavar='SITE_DIR',
        help='Specify the site directory %(metavar)s, should contain a config.cfg')
except:
    pass


def main():
    """
    Called from the handcrank script, this is the main entry point.

    See setup.py for the path that's taken to this function.
    """
    from handcrank.runner import Runner, MissingConfigFile

    if not parser:
        print 'Parser not available, cannot continue'
        sys.exit(1)

    try:
        options = parser.parse_args()
        runner = Runner(options)
        runner.begin()
    except MissingConfigFile as mcf:
        print '%s' % mcf
        sys.exit(1)



from os.path import join

from handcrank.tests.testcase import TestCase
from handcrank import parser
from handcrank.runner import Runner


class RunnerTest(TestCase):
    def test_will_retrieve_delegate(self):
        options = parser.parse_args(['generate'])

        runner = Runner(options)
        delegate = runner.get_delegate_class('Delegate',
           join(self.fixtures, 'site_normal'))

        self.assertEqual('Delegate', delegate.__name__)



from os.path import join, isdir

from handcrank.tests.testcase import TestCase
from handcrank import parser
from handcrank.runner import Runner


class CreateSkeletonTest(TestCase):
    def test_requires_site_directory(self):
        options = parser.parse_args(['startsite'])
        runner = Runner(options)

        with self.assertRaises(SystemExit) as exc_context:
            runner.begin()
        try:
            self.assertEquals(1, exc_context.exception.code)
        except AttributeError:
            self.assertEquals(1, exc_context.exception)

    def test_creates_site_directory(self):
        working = join(self.fixtures, 'working')

        options = parser.parse_args(['--sitedir', working, 'startsite'])
        runner = Runner(options)

        with self.assertRaises(SystemExit) as exc_context:
            runner.begin()
        try:
            self.assertEquals(0, exc_context.exception.code)
        except AttributeError:
            self.assertEquals(0, exc_context.exception)
        self.assertTrue(isdir(working))



from os.path import join

from nose.plugins.attrib import attr

from handcrank.tests.testcase import TestCase
from handcrank import generator
from handcrank import parser
from handcrank.runner import Runner
from handcrank.roles import escape


class BasicGeneratorTest(TestCase):
    def setUp(self):
        super(BasicGeneratorTest, self).setUp()

        self.site_normal = join(self.fixtures, 'site_normal')

    def test_checks_for_arguments(self):
        with self.assertRaises(generator.GeneratorError):
            gen = generator.OnePageGenerator()

        with self.assertRaises(generator.GeneratorError):
            gen = generator.OnePageGenerator(from_dir='.', out_dir='.')

        with self.assertRaises(generator.GeneratorError):
            gen = generator.OnePageGenerator(out_dir='.', tmp_dir='.')

        with self.assertRaises(generator.GeneratorError):
            gen = generator.OnePageGenerator(from_dir='.', tmp_dir='.')

        gen = generator.OnePageGenerator(from_dir='.', tmp_dir='.', out_dir='.')

    def _generate_site_normal(self, ):
        options = parser.parse_args(['--sitedir', self.site_normal, 'generate'])

        runner = Runner(options)
        runner.begin()

    def test_will_create_output_directory(self):
        options = parser.parse_args(['--sitedir', self.site_normal, 'generate'])

        runner = Runner(options)
        runner.begin()

        index = open(join(self.working, 'index.html')).read()

        self.assertIn('Monty Python', index)
        self.assertIn('The quest', index)

    def test_will_render_code_blocks(self):
        options = parser.parse_args(['--sitedir', self.site_normal, 'generate'])

        runner = Runner(options)
        runner.begin()

        index = open(join(self.working, 'index.html')).read()

        self.assertIn('<div class="highlight"><pre><span class="kn">from', index)

    def test_delegate_items_place_first(self):
        delegate = generator.DelegateBase()

        to_test = {'a': 1, 'b': 2, 'c': 3}

        od = delegate.items_place_first(to_test.items(), 'c')

        first = iter(od).next()

        self.assertEquals(('c', 3), first)

    def test_delegate_items_place_first_missing_key(self):
        delegate = generator.DelegateBase()

        to_test = {'a': 1, 'b': 2, 'c': 3}

        with self.assertRaises(generator.GeneratorError) as context:
            od = delegate.items_place_first(to_test.items(), 'd')

            self.assertEqual(str(context.exception),
                'Item d not found, available are: a, b, c')

    def test_delegate_items_sort_alpha(self):
        delegate = generator.DelegateBase()

        to_test = {'moommoo': 1, 'babba': 2, 'abba': 3, 'zeta': 4}

        od = delegate.items_sort_alpha(to_test.items())

        self.assertListEqual(
            [('abba', 3), ('babba', 2), ('moommoo', 1), ('zeta', 4)],
            od)

    def test_delegate_renders_expected(self):
        options = parser.parse_args(['--sitedir', self.site_normal, 'generate'])

        runner = Runner(options)
        runner.begin()

        index = open(join(self.working, 'index.html')).read()

        self.assertIn('Provide extra context', index)

    def test_will_create_doc_link(self):
        options = parser.parse_args(['--sitedir', self.site_normal, 'generate'])

        runner = Runner(options)
        runner.begin()

        index = open(join(self.working, 'index.html')).read()

        self.assertIn(
            '<a href="the_quest.rst" rel="doc-link">you can read that here</a>',
            index)

    def test_html_entity_escape(self):
        self.assertEqual('&amp;&lt;&gt;&quot;&#39;', escape('&<>"\''))



import re

from docutils import nodes, utils
from docutils.parsers.rst import directives
from docutils.parsers.rst.roles import _role_registry

explicit_title_re = re.compile(r'^(.+?)\s*(?<!\x00)<(.*?)>$', re.DOTALL)


def escape(html):
    """
    Escape HTML entities
    """
    html_codes = (
        ('&', '&amp;'),
        ('<', '&lt;'),
        ('>', '&gt;'),
        ('"', '&quot;'),
        ("'", '&#39;'),)

    for code in html_codes:
        html = html.replace(code[0], code[1])

    return html


def split_explicit_title(text):
    """Split role content into title and target, if given."""
    match = explicit_title_re.match(text)
    if match:
        return True, match.group(1), match.group(2)
    return False, text, text


def register_canonical_role(name, role_fn):
    """
    Register an interpreted text role by its canonical name.

    :Parameters:
      - `name`: The canonical name of the interpreted role.
      - `role_fn`: The role function.  See the module docstring.
    """
    set_implicit_options(role_fn)
    _role_registry[name] = role_fn


def doc_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
    """
    Provides a role for converting :doc:`a link <to_this>` into <a
    href="to_this" rel="doc-link">a link</a>
    """
    options['format'] = 'html'
    link_format = '<a href="%s" rel="doc-link">%s</a>'

    matches, title, link = split_explicit_title(text)
    # We want to make sure that the link is "somefile.rst"
    assert '.rst' in link.lower()
    link = link_format % (escape(link), title,)
    node = nodes.raw(rawtext, utils.unescape(link, 1), **options)
    return [node], []


def set_implicit_options(role_fn):
    """
    Add customization options to role functions, unless explicitly set or
    disabled.
    """
    if not hasattr(role_fn, 'options') or role_fn.options is None:
        role_fn.options = {'class': directives.class_option}
    elif 'class' not in role_fn.options:
        role_fn.options['class'] = directives.class_option

doc_role.options = {'format': directives.unchanged}

register_canonical_role('doc', doc_role)



from docutils import nodes
from docutils.parsers.rst import Directive, directives

from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import HtmlFormatter


class HtmlCodeBlock(Directive):
    """
    *Borrowed from the Sphinx project* (Alright, stolen)

    Docutils does not natively understand a ..code-bock:: directive.  Sphinx
    adds this so that you can get Pygments syntax highlighting of literal
    blocks.

    Sphinx does this in such a way that it can be converted to XML or Latex.
    This is not necessary for our purposes, so we are using a Raw docutils node
    at the end instead of returning a more agnostic node.

    Setting the format to `html` allows the html writer to output the contents.
    """

    has_content = True
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = False
    option_spec = {
        'linenos': directives.flag,
    }

    def run(self):
        code = u'\n'.join(self.content)

        linenos = 'linenos' in self.options
        lexer = get_lexer_by_name(self.arguments[0])
        formatter = HtmlFormatter(linenos=linenos)

        hi_code = highlight(code, lexer, formatter)

        raw = nodes.raw('', hi_code, **{'format': 'html', 'source': None})
        return [raw]

directives.register_directive('code-block', HtmlCodeBlock)



import sys
import shutil
import itertools
from os import listdir
from os.path import isfile, isdir, join
from docutils.core import publish_parts

from jinja2 import Template

from handcrank.report import Report

# Imported to support additional directives and rolls
from handcrank import directives
from handcrank import roles

# OrderedDict import
try:
    # Python >= 2.7 / 3.1
    from collections import OrderedDict
except ImportError:
    # Python < 2.7
    from ordereddict import OrderedDict


class GeneratorError(Exception):
    pass


class DelegateBase(object):
    """
    Used along side Generator classes to alter the behavior of the generated
    content
    """
    def items_place_first(self, source_list, *args):
        """
        Allows you to place the keys specified by *args first in the source
        list.

        For example if you had this:
            source_list = [
                ('john', 'cleese'), ('eric', 'idle'), ('terry', 'jones')]

        Calling items_place_first(source_list, 'terry') would result in:
            source_list = [
                ('terry', 'jones'), ('john', 'cleese'), ('eric', 'idle')]
        """
        sldict = {}
        for basename, source in source_list:
            sldict[basename] = source

        od = OrderedDict()
        for item in args:
            try:
                od[item] = sldict.pop(item)
            except KeyError:
                available = sldict.keys()
                available.sort()
                raise GeneratorError('Item %s not found, available are: %s' %
                    (item, ', '.join(available)))

        for item in sldict:
            od[item] = sldict[item]

        return od.items()

    def items_sort_alpha(self, source_list):
        """
        Alphabetizes the source_list by key
        """
        source_list = list(source_list)
        source_list.sort(lambda x, y: cmp(x[0].lower(), y[0].lower()))
        return source_list

    def provide_extra_context(self):
        """
        Specify, by returning a dictionary, any extra context variables you
        need to render this site
        """
        return {}

    def sort_source_list(self, source_list):
        """
        Allows the list or source files to be sorted differently
        """
        return self.items_sort_alpha(source_list)


class SourceList():
    """
    Converts a directory of files into a dictionary of::

        { 'FILENAME': 'FILE CONTENTS' }
    """
    def __init__(self, dir, filter_extension='rst'):
        self.dir = dir
        self.extension = filter_extension

    def __iter__(self):
        return iter(self.open())

    def open(self):
        if not isdir(self.dir):
            raise GeneratorError('%s is not a directory' % self.dir)

        for path in listdir(self.dir):
            filename = join(self.dir, path)
            if not isfile(filename):
                continue
            if not path.endswith(self.extension):
                continue
            with open(filename) as fh:
                yield path, fh.read()


class OnePageGenerator():
    """
    Takes a directory ``from_dir`` of rST files and creates a single page
    website from it into ``out_dir`` using the template in ``tmp_dir``

        from_dir      The directory that contans the rST files
        out_dir       Where you want the generator to create the site
        tmp_dir       The template directory, usually called 'template'
    """
    delegate = DelegateBase()

    def __init__(self, **options):
        """
        Required parameters: from_dir, out_dir, tmp_dir
        """
        self.options = options

        try:
            assert self.options['from_dir']
            assert self.options['out_dir']
            assert self.options['tmp_dir']
        except (AssertionError, KeyError):
            raise GeneratorError(self.__init__.__doc__)

        for f in ('template_file', 'output_file',):
            if not f in self.options:
                self.options[f] = 'index.html'

        self.source_list = SourceList(options['from_dir'])
        self.indicator = None

    def tick(self):
        """
        Writes an indicator to sys.out that lets the user know something is
        happening
        """
        if not self.indicator:
            self.indicator = itertools.cycle('-_')

        sys.stdout.write(self.indicator.next())
        sys.stdout.flush()

    def turn_handle(self):
        """
        Runs through all the rST files and creates the output files based on
        the template in tmp_dir
        """
        gathering, source_count = self.convert_to_html()

        shutil.rmtree(self.options['out_dir'], ignore_errors=True)
        shutil.copytree(self.options['tmp_dir'], self.options['out_dir'])

        context = {
            'gathering': gathering}

        context.update(self.delegate.provide_extra_context())

        generated_count = 0
        
        with open(join(self.options['tmp_dir'],
            self.options['template_file'])) as fh:
            template = Template(fh.read())

            with open(join(self.options['out_dir'],
                self.options['output_file']), 'w') as wfh:
                generated_count += 1
                wfh.write(template.render(**context))

        return Report(
            from_dir=self.options['from_dir'],
            out_dir=self.options['out_dir'],
            tmp_dir=self.options['tmp_dir'],
            generated_count=generated_count,
            source_count=source_count,
            entry_point=None)

    def convert_to_html(self, docutils_settings={}):
        """
        Converts the source rST files to HTML.

        Return a tuple of dictiory, integer where the dictionary has the title
        as it's key and the HTML source as it's value.  The second element in
        the tuple is an integer representing the number of source files
        rendered.
        """
        source_count = 0
        gathering = OrderedDict()

        sl_srt = self.delegate.sort_source_list

        for basename, source in sl_srt(self.source_list):
            parts = publish_parts(
                source=source,
                writer_name="html4css1",
                settings_overrides=docutils_settings)

            parts['basename'] = basename

            if parts['title'] in gathering:
                raise GeneratorError(
                    'Title %s already present for %s' % (
                         parts['title'], basename))

            gathering[parts['title']] = parts
            self.tick()
            source_count += 1

        return gathering, source_count



from handcrank.tests.testcase import TestCase
from handcrank.report import Report


class ReportTest(TestCase):
    def test_can_create_and_dump(self):
        Report(
            from_dir='/somewhere',
            tmp_dir='/somewhere',
            out_dir='/somewhere',
            generated_count=0,
            source_count=0,
            entry_point=None).dump()



"""
This Report class is used in conjunction with the Generator class.

After the Generator performs is tasks, the information contained within it
should be given to the user in a digestable form that allows them to decide
what the next course of action is.

This report formats the data the generator has been reponsible for creating.

By making this object responsible for output regarding the results of the
generation, we can let the Generator class do it's work and pass it's results
off to another object.  This keeps the output uniform, and prevents duplication
of this feature.
"""
import textwrap
import collections

Report = collections.namedtuple('Report',
    'from_dir out_dir tmp_dir source_count generated_count entry_point')

def dump(self):
    print textwrap.dedent("""
    Source rST files .......... (%(source_count)s files) %(from_dir)s 
    Used the template ......... %(tmp_dir)s
    Created ................... %(out_dir)s
    """ % self._asdict())

setattr(Report, 'dump', dump)



from os.path import join, dirname
import shutil

# Backward compatible unittest
try:
    import unittest
    # skip was added in 2.7/3.1
    assert unittest.skip
except AttributeError:
    import unittest2 as unittest


class TestCase(unittest.TestCase):
    def setUp(self):
        self.here = dirname(__file__)
        self.fixtures = join(self.here, 'fixtures')
        self.working = join(self.fixtures, 'working')

        shutil.rmtree(self.working, ignore_errors=True)



from setuptools import setup, find_packages
import sys, os

# So we can import handcrank
sys.path.append('./src')

from handcrank import version

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
NEWS = open(os.path.join(here, 'NEWS.rst')).read()

install_requires = [
    'docutils>=0.7',
    'Jinja2==2.5.2',
    'Pygments>=0.8',
]

pyversion = sys.version_info
if pyversion[0] == 2 and pyversion[1] < 7:
    # We need to include argparse, it's not part of the standard lib
    install_requires.append('argparse>=1.1')
    install_requires.append('ordereddict>=1.1')
    install_requires.append('unittest2>=0.5.1')

setup(name='handcrank',
    version=version.number,
    description="A very simple static site generator using reStructuredText",
    long_description=README + '\n\n' + NEWS,
    classifiers=[
      # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    ],
    keywords='rst restructuredtext site generator',
    author='Rob Madole',
    author_email='robmadole@gmail.com',
    url='http://github.com/robmadole/handcrank',
    license='BSD',
    packages=find_packages('src'),
    package_dir = {'': 'src'},include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    entry_points={
        'console_scripts':
            ['handcrank=handcrank:main']
    }
)



from os import walk, stat
from os.path import isdir, join
import UserDict


class Snapshot(UserDict.IterableUserDict):
    def __init__(self, ignore=[]):
        UserDict.IterableUserDict.__init__(self)

        assert not type(ignore) in (basestring,)
        self.ignore = ignore

    def __eq__(self, compared_with):
        if not isinstance(compared_with, Snapshot):
            return False

        if not len(self) is len(compared_with):
            return False

        for item in self:
            if [i for i in self.ignore if i in item]:
                # This item is being ignored
                continue
            if item not in compared_with:
                return False
            if not self[item] == compared_with[item]:
                return False

        return True


class Monitor(object):
    """
    Looks at a directory and tells you if anything has changed within it
    """
    def __init__(self, directory, ignore_directory=[]):
        if not isdir(directory):
            raise IOError('%s is not a directory, cannot monitor it for '
                'changes' % directory)

        self.directory = directory
        self.snapshot = None
        self.ignore_directory = ignore_directory

    def has_changed(self):
        older_snapshot = self.snapshot

        self.snapshot = Snapshot(ignore=self.ignore_directory)

        for (dirpath, dirnames, filenames) in walk(self.directory):
            for basename in filenames:
                filename = join(dirpath, basename)
                lastmod = stat(filename).st_mtime
                self.snapshot[filename] = lastmod

        if older_snapshot == self.snapshot or older_snapshot is None:
            return False

        return True



import time
import shutil
from os import unlink, mkdir
from os.path import join

from handcrank.tests.testcase import TestCase
from handcrank.dirchange import Monitor


class DirectoryChangeTest(TestCase):
    def test_handles_bad_directory(self):
        with self.assertRaises(IOError):
            monitor = Monitor('/somethingthatdoesnotexist')

    def assertChanged(self, monitor):
        # First time should be yep
        self.assertTrue(monitor.has_changed())
        # Calling it again (because nothing has changed) should be nope
        self.assertFalse(monitor.has_changed())

    def assertNotChanged(self, monitor):
        self.assertFalse(monitor.has_changed())

    def test_will_detect_new_file(self):
        shutil.copytree(join(self.fixtures, 'site_normal'), self.working)
        monitor = Monitor(self.working)

        self.assertNotChanged(monitor)

        open(join(self.working, 'knights_ni.rst'), 'w').close()

        self.assertChanged(monitor)

    def test_will_detect_file_deleted(self):
        shutil.copytree(join(self.fixtures, 'site_normal'), self.working)
        monitor = Monitor(self.working)

        open(join(self.working, 'knights_ni.rst'), 'w').close()
        open(join(self.working, 'the_french.rst'), 'w').close()
        open(join(self.working, 'three_headed_knight.rst'), 'w').close()
        open(join(self.working, 'template', 'index.html'), 'w').close()

        self.assertNotChanged(monitor)

        unlink(join(self.working, 'knights_ni.rst'))

        self.assertChanged(monitor)

        unlink(join(self.working, 'template', 'index.html'))

        self.assertChanged(monitor)

    def test_will_detect_file_modified(self):
        shutil.copytree(join(self.fixtures, 'site_normal'), self.working)
        monitor = Monitor(self.working)

        self.assertNotChanged(monitor)

        time.sleep(1.0)
        with open(join(self.working, 'the_quest.rst'), 'w') as fh:
            fh.write('Clippity cloppity clippity cloppity coconut')

        self.assertChanged(monitor)

        time.sleep(1.0)
        with open(join(self.working, 'template', 'index.html'), 'w') as fh:
            fh.write('<htNI!l><htNI!l>')

        self.assertChanged(monitor)

    def test_will_ignore_directories(self):
        shutil.copytree(join(self.fixtures, 'site_normal'), self.working)
        mkdir(join(self.working, 'output'))

        with open(join(self.working, 'output', 'the_quest.html'), 'w') as fh:
            fh.write('<html>Clippity cloppity coconut</html>')

        monitor = Monitor(self.working)

        self.assertNotChanged(monitor)

        time.sleep(1.0)
        open(join(self.working, 'output', 'the_quest.html'), 'w').close()

        self.assertChanged(monitor)

        # Now we ignore the output directory
        monitor = Monitor(self.working,
            ignore_directory=join(self.working, 'output'))

        self.assertNotChanged(monitor)

        time.sleep(1.0)
        open(join(self.working, 'output', 'the_quest.html'), 'w').close()

        self.assertNotChanged(monitor)
